@@ -10,20 +10,24 @@ import pandas as pd
 import xarray as xr
 
 from .prepare import (
-    _parse_fn_args,
-    _parse_var_names,
-    _parse_var_dims,
-    _parse_var_coords,
-    _parse_constants,
-    _parse_resources,
-    _parse_combos,
-    _parse_cases,
-    _parse_attrs,
+    parse_fn_args,
+    parse_var_names,
+    parse_var_dims,
+    parse_var_coords,
+    parse_constants,
+    parse_resources,
+    parse_combos,
+    parse_cases,
+    parse_attrs,
 )
 from .combo_runner import combo_runner_to_ds
 from .case_runner import case_runner_to_ds
 from ..manage import load_ds, save_ds, load_df, save_df
-from . import batch
+from . import cropping
+
+
+class XYZError(Exception):
+    pass
 
 
 # --------------------------------------------------------------------------- #
@@ -71,13 +75,14 @@ class Runner(object):
                  attrs=None,
                  **default_runner_settings):
         self.fn = fn
-        self._var_names = _parse_var_names(var_names)
-        self._fn_args = _parse_fn_args(fn, fn_args)
-        self._var_dims = _parse_var_dims(var_dims, self._var_names)
-        self._var_coords = _parse_var_coords(var_coords)
-        self._constants = _parse_constants(constants)
-        self._resources = _parse_resources(resources)
-        self._attrs = _parse_attrs(attrs)
+        self._var_names = parse_var_names(var_names)
+        self._fn_args = parse_fn_args(fn, fn_args)
+        self._var_dims = parse_var_dims(var_dims, self._var_names)
+        self._var_coords = parse_var_coords(var_coords)
+        self._constants = parse_constants(constants)
+        self._resources = parse_resources(resources)
+        self._attrs = parse_attrs(attrs)
+        self._last_ds = None
         self.default_runner_settings = default_runner_settings
 
     def __call__(self, *args, **kwargs):
@@ -89,7 +94,7 @@ class Runner(object):
         return self._fn_args
 
     def _set_fn_args(self, fn_args):
-        self._fn_args = _parse_fn_args(self.fn, fn_args)
+        self._fn_args = parse_fn_args(self.fn, fn_args)
 
     def _del_fn_args(self):
         self._fn_args = None
@@ -101,7 +106,7 @@ class Runner(object):
         return self._var_names
 
     def _set_var_names(self, var_names):
-        self._var_names = _parse_var_names(var_names)
+        self._var_names = parse_var_names(var_names)
 
     def _del_var_names(self):
         self._var_names = None
@@ -115,7 +120,7 @@ class Runner(object):
     def _set_var_dims(self, var_dims, var_names=None):
         if var_names is None:
             var_names = self._var_names
-        self._var_dims = _parse_var_dims(var_dims, var_names)
+        self._var_dims = parse_var_dims(var_dims, var_names)
 
     def _del_var_dims(self):
         self._var_dims = None
@@ -127,7 +132,7 @@ class Runner(object):
         return self._var_coords
 
     def _set_var_coords(self, var_coords):
-        self._var_coords = _parse_var_coords(var_coords)
+        self._var_coords = parse_var_coords(var_coords)
 
     def _del_var_coords(self):
         self._var_coords = None
@@ -139,7 +144,7 @@ class Runner(object):
         return self._constants
 
     def _set_constants(self, constants):
-        self._constants = _parse_constants(constants)
+        self._constants = parse_constants(constants)
 
     def _del_constants(self):
         self._constants = None
@@ -151,7 +156,7 @@ class Runner(object):
         return self._resources
 
     def _set_resources(self, resources):
-        self._resources = _parse_resources(resources)
+        self._resources = parse_resources(resources)
 
     def _del_resources(self):
         self._resources = None
@@ -160,6 +165,10 @@ class Runner(object):
                          "Runner's function that are *not* saved with the "
                          "dataset.")
 
+    @property
+    def last_ds(self):
+        return self._last_ds
+
     # Running methods ------------------------------------------------------- #
 
     def run_combos(self, combos, constants=(), **runner_settings):
@@ -167,18 +176,20 @@ class Runner(object):
 
         Parameters
         ----------
-        combos : tuple of form ((str, seq), ...)
+        combos : dict_like
             The values of each function argument with which to evaluate all
             combinations.
-        constants : dict (optional)
+        constants : dict, optional
             Extra constant arguments for this run, repeated arguments will
             take precedence over stored constants but for this run only.
         runner_settings
             Keyword arguments supplied to :func:`~xyzpy.combo_runner`.
         """
-        combos = _parse_combos(combos)
-        self.last_ds = combo_runner_to_ds(
-            self.fn, combos, self._var_names,
+        combos = parse_combos(combos)
+        self._last_ds = combo_runner_to_ds(
+            fn=self.fn,
+            combos=combos,
+            var_names=self._var_names,
             var_dims=self._var_dims,
             var_coords=self._var_coords,
             constants={**self._constants, **dict(constants)},
@@ -186,7 +197,7 @@ class Runner(object):
             attrs=self._attrs,
             parse=False,
             **{**self.default_runner_settings, **runner_settings})
-        return self.last_ds
+        return self._last_ds
 
     def run_cases(self, cases, constants=(), fn_args=None, **runner_settings):
         """Run cases using the function and save to dataset.
@@ -201,12 +212,12 @@ class Runner(object):
         runner_settings
             Supplied to :func:`~xyzpy.case_runner`.
         """
-        cases = _parse_cases(cases)
-
         if fn_args is None:
             fn_args = self._fn_args
 
-        self.last_ds = case_runner_to_ds(
+        cases = parse_cases(cases, fn_args)
+
+        self._last_ds = case_runner_to_ds(
             fn=self.fn,
             fn_args=fn_args,
             cases=cases,
@@ -218,7 +229,7 @@ class Runner(object):
             attrs=self._attrs,
             parse=False,
             **{**self.default_runner_settings, **runner_settings})
-        return self.last_ds
+        return self._last_ds
 
     def Crop(self,
              name=None,
@@ -234,12 +245,9 @@ class Runner(object):
         -------
         Crop
         """
-        return batch.Crop(farmer=self,
-                          name=name,
-                          parent_dir=parent_dir,
-                          save_fn=save_fn,
-                          batchsize=batchsize,
-                          num_batches=num_batches)
+        return cropping.Crop(farmer=self, name=name, parent_dir=parent_dir,
+                             save_fn=save_fn, batchsize=batchsize,
+                             num_batches=num_batches)
 
     def __repr__(self):
         string = "<xyzpy.Runner>\n"
@@ -271,8 +279,11 @@ def label(var_names,
           constants=None,
           resources=None,
           attrs=None,
+          harvester=False,
+          sampler=False,
           **default_runner_settings):
-    """Decorator to automatically wrap a function as a :class:`~xyzpy.Runner`.
+    """Convenient decorator to automatically wrap a function as a
+    :class:`~xyzpy.Runner` or :class:`~xyzpy.Harvester`.
 
     Parameters
     ----------
@@ -296,6 +307,9 @@ def label(var_names,
         Like `constants` but not saved to the the dataset, e.g. if very big.
     attrs : dict-like, optional
         Any other miscelleous information to be saved with the dataset.
+    harvester : bool or str, optional
+        If ``True``, wrap the runner as a :class:`~xyzpy.Harvester`, if a
+        string, create the harvester with that as the ``data_name``.
     default_runner_settings
         These keyword arguments will be supplied as defaults to any runner.
 
@@ -322,10 +336,29 @@ def label(var_names,
         (3, -1)
 
     """
+    if harvester and sampler:
+        raise ValueError("Cannot be both a harvester and a sampler.")
+
     def wrapper(fn):
+
         r = Runner(fn, var_names, fn_args=fn_args, var_dims=var_dims,
                    var_coords=var_coords, constants=constants,
                    resources=resources, attrs=attrs, **default_runner_settings)
+
+        if harvester:
+            if harvester is True:
+                data_name = None
+            else:
+                data_name = harvester
+            r = Harvester(r, data_name=data_name)
+
+        if sampler:
+            if sampler is True:
+                data_name = None
+            else:
+                data_name = sampler
+            r = Sampler(r, data_name=data_name)
+
         return functools.update_wrapper(r, fn)
 
     return wrapper
@@ -378,6 +411,9 @@ class Harvester(object):
     def fn(self, fn):
         self.runner.fn = fn
 
+    def __call__(self, *args, **kwargs):
+        return self.fn(*args, **kwargs)
+
     @property
     def last_ds(self):
         """Dataset containing the last runs' data.
@@ -435,6 +471,10 @@ class Harvester(object):
         engine : str, optional
             Engine to use to save and load datasets.
         """
+        if self.data_name is None:
+            raise XYZError("You didn't set a ``data_name`` for this harvester "
+                           "to use for persistent storage.")
+
         if engine is None:
             engine = self.engine
 
@@ -537,18 +577,62 @@ class Harvester(object):
         else:
             self._full_ds = new_full_ds
 
-    def harvest_combos(self, combos, *,
-                       sync=True,
-                       overwrite=None,
-                       chunks=None,
-                       engine=None,
-                       **runner_settings):
+    def expand_dims(
+        self,
+        name,
+        value,
+        engine=None,
+    ):
+        """Add a new coordinate dimension with ``name`` and ``value``. The
+        change is immediately synced with the on-disk dataset. Useful if you
+        want to expand the parameter space along a previously constant
+        argument.
+        """
+        new_ds = self.full_ds.expand_dims(name)
+        new_ds.coords[name] = [value]
+        if self.data_name is not None:
+            self.save_full_ds(new_ds, engine=engine)
+        else:
+            self._full_ds = new_ds
+
+    def drop_sel(
+        self,
+        labels=None,
+        *,
+        errors='raise',
+        engine=None,
+        **labels_kwargs,
+    ):
+        """Drop specific values of coordinates from this harvester and its
+        dataset. See
+        http://xarray.pydata.org/en/latest/generated/xarray.Dataset.drop_sel.html.
+        The change is immediately synced with the on-disk dataset.
+        Useful for tidying uneeded data points.
+        """
+        new_ds = self.full_ds.drop_sel(labels, errors=errors, **labels_kwargs)
+        if self.data_name is not None:
+            self.save_full_ds(new_ds, engine=engine)
+        else:
+            self._full_ds = new_ds
+
+    def harvest_combos(
+        self,
+        combos,
+        *,
+        sync=True,
+        overwrite=None,
+        chunks=None,
+        engine=None,
+        **runner_settings
+    ):
         """Run combos, automatically merging into an on-disk dataset.
 
         Parameters
         ---------
-        combos : mapping_like
-            The combos to run.
+        combos : dict_like
+            The combos to run. The only difference here is that you can supply
+            an ellipse ``...``, meaning the all values for that coordinate will
+            be loaded from the current full dataset.
         sync : bool, optional
             If True (default), load and save the disk dataset before
             and after merging in the new data.
@@ -568,7 +652,10 @@ class Harvester(object):
         runner_settings
             Supplied to :func:`~xyzpy.combo_runner`.
         """
-
+        combos = tuple(
+            (key, self.full_ds.coords[key].values if values is ... else values)
+            for key, values in parse_combos(combos)
+        )
         ds = self.runner.run_combos(combos, **runner_settings)
         self.add_ds(ds, sync=sync, overwrite=overwrite,
                     chunks=chunks, engine=engine)
@@ -623,12 +710,9 @@ class Harvester(object):
         -------
         Crop
         """
-        return batch.Crop(farmer=self,
-                          name=name,
-                          parent_dir=parent_dir,
-                          save_fn=save_fn,
-                          batchsize=batchsize,
-                          num_batches=num_batches)
+        return cropping.Crop(farmer=self, name=name, parent_dir=parent_dir,
+                             save_fn=save_fn, batchsize=batchsize,
+                             num_batches=num_batches)
 
     def __repr__(self):
         string = ("<xyzpy.Harvester>\n"
@@ -794,8 +878,13 @@ class Sampler:
         )
         return tuple(combos.keys()), cases
 
-    def sample_combos(self, n, combos=None, engine=None,
-                      **case_runner_settings):
+    def sample_combos(
+        self,
+        n,
+        combos=None,
+        engine=None,
+        **case_runner_settings,
+    ):
         """Sample the target function many times, randomly choosing parameter
         combinations from ``combos`` (or ``SampleHarvester.default_combos``).
 
@@ -821,12 +910,14 @@ class Sampler:
         self.add_df(last_df, engine=engine)
         return last_df
 
-    def Crop(self,
-             name=None,
-             parent_dir=None,
-             save_fn=None,
-             batchsize=None,
-             num_batches=None):
+    def Crop(
+        self,
+        name=None,
+        parent_dir=None,
+        save_fn=None,
+        batchsize=None,
+        num_batches=None,
+    ):
         """Return a Crop instance with this Sampler, from which `fn`
         will be set, and then samples can be sown, grown, and reaped into the
         ``Sampler.full_df``. See :class:`~xyzpy.Crop`.
@@ -835,12 +926,9 @@ class Sampler:
         -------
         Crop
         """
-        return batch.Crop(farmer=self,
-                          name=name,
-                          parent_dir=parent_dir,
-                          save_fn=save_fn,
-                          batchsize=batchsize,
-                          num_batches=num_batches)
+        return cropping.Crop(farmer=self, name=name, parent_dir=parent_dir,
+                             save_fn=save_fn, batchsize=batchsize,
+                             num_batches=num_batches)
 
     def __repr__(self):
         string = ("<xyzpy.Sampler>\n"
